package de.hft.stuttgart.citydoctor2.gui;

import de.hft.stuttgart.citydoctor2.datastructure.*;
import de.hft.stuttgart.citydoctor2.gui.tree.FeatureNode;
import de.hft.stuttgart.citydoctor2.utils.Localization;
import javafx.scene.input.Clipboard;
import javafx.scene.input.ClipboardContent;
import org.locationtech.proj4j.ProjCoordinate;

import de.hft.stuttgart.citydoctor2.gui.tree.Renderable;
import de.hft.stuttgart.citydoctor2.parser.ParserConfiguration;
import javafx.scene.control.ContextMenu;
import javafx.scene.control.CustomMenuItem;
import javafx.scene.control.Label;
import javafx.scene.control.MenuItem;
import javafx.scene.control.TreeView;
import javafx.scene.input.MouseButton;
import javafx.scene.input.MouseEvent;
import javafx.stage.Stage;

import static javafx.scene.input.Clipboard.getSystemClipboard;

public class VertexClickHandler implements ClickHandler {
	
	private TreeView<Renderable> errorView;
	private Renderer renderer;
	private Stage stage;
	private MainWindow mainWindow;
	private ParserConfiguration config;
	
	public VertexClickHandler(TreeView<Renderable> errorView, Renderer renderer, Stage stage, MainWindow mainWindow) {
		this.errorView = errorView;
		this.renderer = renderer;
		this.stage = stage;
		this.mainWindow = mainWindow;
	}
	
	public void setConfig(ParserConfiguration config) {
		this.config = config;
	}
	
	@Override
	public void onPolygonClick(Polygon p, MouseEvent me) {
		if (me.getButton() == MouseButton.PRIMARY) {
			errorView.getSelectionModel().clearSelection();
			renderer.highlight(p);
		} else if (me.getButton() == MouseButton.SECONDARY) {
			ContextMenu cMenu = new ContextMenu();

			MenuItem clipMi = new MenuItem(Localization.getText("MainWindow.copyId"));
			clipMi.setOnAction(ea -> {
				CityObject topLevelCityObject = p.getParent().getParent().getTopLevelCityObject();
				Clipboard clipboard = getSystemClipboard();
				ClipboardContent content = new ClipboardContent();
				content.putString(topLevelCityObject.getGmlId().toString());
				clipboard.setContent(content);
			});
			cMenu.getItems().add(clipMi);

			MenuItem detailMi = new MenuItem(Localization.getText("MainWindow.focusCityObject"));
			detailMi.setOnAction(ea -> {
				CityObject topLevelCityObject = p.getParent().getParent().getTopLevelCityObject();
				mainWindow.showCityObjectDetail(topLevelCityObject);

			});
			cMenu.getItems().add(detailMi);

			cMenu.show(stage, me.getScreenX(), me.getScreenY());
		}
	}

	@Override
	public void onVertexClick(Vertex v, MouseEvent me) {
		if (me.getButton() == MouseButton.SECONDARY) {
			MenuItem mi1 = new CustomMenuItem(new Label("Vertex"));
			double x = v.getX();
			double y = v.getY();
			if (config.getOriginalTransform() != null) {
				ProjCoordinate p1 = new ProjCoordinate();
				ProjCoordinate p2 = new ProjCoordinate();
				p1.x = v.getX();
				p1.y = v.getY();
				config.getOriginalTransform().transform(p1, p2);
				x = p2.x;
				y = p2.y;
			}
			
			MenuItem mi2 = new MenuItem("x = " + x);
			MenuItem mi3 = new MenuItem("y = " + y);
			MenuItem mi4 = new MenuItem("z = " + v.getZ());
			ContextMenu cMenu = new ContextMenu(mi1, mi2, mi3, mi4);
			cMenu.show(stage, me.getScreenX(), me.getScreenY());
		}
	}

}
